from pathlib import Path

DEFAULT_SERVER_CONFIG_PATH = str(Path("~/.askanna.yml").expanduser())
DEFAULT_SERVER_REMOTE = "https://beta-api.askanna.eu"
DEFAULT_SERVER_UI = "https://beta.askanna.eu"

DEFAULT_PROJECT_TEMPLATE = "https://gitlab.com/askanna/project-templates/blanco-template.git"

PYPI_PROJECT_URL = "https://pypi.org/pypi/askanna/json"
